@@ -1,8 +1,6 @@
 package HPC::Runner;
 
-#use 5.006;
-
-use Carp::Always;
+#use Carp::Always;
 use Data::Dumper;
 use IPC::Open3;
 use IO::Select;
@@ -13,7 +11,6 @@ use DateTime::Format::Duration;
 use Cwd;
 use File::Path qw(make_path);
 use File::Spec;
-#use String::Escape qw( quote  );
 
 use Moose;
 use namespace::autoclean;
@@ -22,6 +19,7 @@ use Moose::Util::TypeConstraints;
 #with 'MooseX::Getopt';
 with 'MooseX::Getopt::Usage';
 with 'MooseX::Getopt::Usage::Role::Man';
+with 'MooseX::Object::Pluggable';
 
 # For pretty man pages!
 $ENV{TERM}='xterm-256color';
@@ -36,7 +34,7 @@ Version 2.4.2
 
 =cut
 
-our $VERSION = '2.45';
+our $VERSION = '2.46';
 
 =head1 SYNOPSIS
 
@@ -102,7 +100,6 @@ Internal variable
 sub _set_outdir{
     my($self, $outdir) = @_;
 
-    #make_path(quote($outdir)) if ! -d $outdir;
     make_path($outdir) if -d $outdir;
     $outdir = File::Spec->rel2abs($outdir);
     $self->{outdir} = $outdir;
@@ -261,7 +258,6 @@ has 'counter' => (
 has 'log' => (
     traits  => ['NoGetopt'],
     is => 'rw',
-#    default => \&init_log,
 );
 
 has 'command_log' => (
@@ -308,8 +304,28 @@ has 'process_table' => (
     lazy => 1,
 );
 
+=head2 plugins
+
+Load plugins
+
+=cut
+
+has 'plugins' => (
+    is => 'rw',
+    isa => 'ArrayRef|Str',
+    documentation => 'Add Plugins to your run',
+);
+
 =head1 Subroutines
 
+=cut
+
+sub BUILD {
+    my $self = shift;
+
+    $self->process_plugins;
+}
+
 =head2 set_logdir
 
 Set the log directory
@@ -320,10 +336,6 @@ sub set_logdir{
     my $self = shift;
 
     my $logdir;
-    use Carp::Always;
-    use Data::Dumper;
-    #confess(Dumper($self));
-    #$logdir = $self->outdir."/".$self->logname."_".$self->set_logfile;
     $logdir = $self->outdir."/".$self->set_logfile."-".$self->logname;
 
     $DB::single=2;
@@ -369,6 +381,34 @@ sub init_log {
     return $log;
 }
 
+=head2 process_plugins
+
+Split and process plugins
+
+=cut
+
+sub process_plugins{
+    my $self = shift;
+
+    return unless $self->plugins;
+
+    if(ref($self->plugins)){
+        my @plugins = @{$self->plugins};
+        foreach my $plugin (@plugins){
+            if($plugin =~ m/,/){
+                my @tmp = split(',', $plugin);
+                foreach my $tmp (@tmp){
+                    push(@plugins, $tmp);
+                }
+            }
+        }
+        $self->load_plugins(@plugins);
+    }
+    else{
+        $self->load_plugin($self->plugins);
+    }
+}
+
 =head2 run_command_threads
 
 Start the thread, run the command, and finish the thread
@@ -431,14 +471,9 @@ sub _log_commands {
 
     $DB::single=2;
 
-    $self->name_log($pid);
-    $DB::single=2;
-
-    $self->command_log($self->init_log);
-
     my($cmdpid, $exitcode) = $self->log_job;
 
-    $self->log_messages("debug", "Finishing job ".$self->counter." with ExitCode $exitcode", $cmdpid);
+    $self->log_cmd_messages("info", "Finishing job ".$self->counter." with ExitCode $exitcode", $cmdpid);
 
     my $dt2 = DateTime->now();
     my $duration = $dt2 - $dt1;
@@ -446,7 +481,7 @@ sub _log_commands {
         pattern => '%Y years, %m months, %e days, %H hours, %M minutes, %S seconds'
     );
 
-    $self->log_messages("info", "Total execution time ".$format->format_duration($duration), $cmdpid);
+    $self->log_cmd_messages("info", "Total execution time ".$format->format_duration($duration), $cmdpid);
 
     $self->log_table($cmdpid, $exitcode, $format->format_duration($duration));
     return $exitcode;
@@ -457,7 +492,8 @@ sub name_log {
     my $pid = shift;
 
     $self->logfile($self->set_logfile);
-    $self->append_logfile("-CMD".$self->counter."-PID_$pid.log");
+    my $string = sprintf ("%03d", $self->counter);
+    $self->append_logfile("-CMD_".$string.".log");
 }
 
 ##TODO extend this in HPC-Runner-Web for ENV tags
@@ -493,12 +529,12 @@ sub log_job {
     }
     $infh->autoflush();
 
+    $self->name_log($cmdpid);
+    $self->command_log($self->init_log);
+
     $DB::single=2;
 
-
-    $self->log_messages("debug", "Starting Job: ".$self->counter." \nCmd is ".$self->cmd, $cmdpid);
-
-    $self->command_log->debug("@ is ".$@) if $@;
+    $self->log_cmd_messages("info", "Starting Job: ".$self->counter." \nCmd is ".$self->cmd, $cmdpid);
 
     $DB::single=2;
 
@@ -512,7 +548,7 @@ sub log_job {
             my $len = sysread $fh, $line, 4096;
             if(not defined $len){
                 # There was an error reading
-                $self->log_messages("fatal", "Error from child: $!" , $cmdpid)
+                $self->log_cmd_messages("fatal", "Error from child: $!" , $cmdpid)
             } elsif ($len == 0){
                 # Finished reading from this FH because we read
                 # 0 bytes.  Remove this handle from $sel.
@@ -520,11 +556,11 @@ sub log_job {
                 next;
             } else { # we read data alright
                 if($fh == $outfh) {
-                    $self->log_messages("info", $line, $cmdpid)
+                    $self->log_cmd_messages("info", $line, $cmdpid)
                 } elsif($fh == $errfh) {
-                    $self->log_messages("error", $line, $cmdpid)
+                    $self->log_cmd_messages("error", $line, $cmdpid)
                 } else {
-                    $self->command_log->fatal("Shouldn't be here!\n");
+                    $self->log_cmd_messages('fatal', "Shouldn't be here!\n");
                 }
             }
         }
@@ -536,10 +572,10 @@ sub log_job {
     return($cmdpid, $exitcode);
 }
 
-sub log_messages{
+sub log_cmd_messages{
     my($self, $level, $message, $cmdpid)  = @_;
 
-    if($self->show_processid){
+    if($self->show_processid && $cmdpid){
         $self->command_log->$level("PID: $cmdpid\t$message");
     }
     else{
@@ -547,8 +583,14 @@ sub log_messages{
     }
 }
 
+sub log_main_messages{
+    my($self, $level, $message)  = @_;
+
+    $self->log->$level($message);
+}
 
 __PACKAGE__->meta->make_immutable;
+
 1;
 
 =head1 AUTHOR
@@ -558,12 +600,9 @@ Jillian Rowe, C<< <jillian.e.rowe at gmail.com> >>
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-runner-init at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Runner-Init>.  I will be notified, and then you'll
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=HPC-Runner>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
@@ -576,19 +615,19 @@ You can also look for information at:
 
 =item * RT: CPAN's request tracker (report bugs here)
 
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Runner-Init>
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=HPC-Runner>
 
 =item * AnnoCPAN: Annotated CPAN documentation
 
-L<http://annocpan.org/dist/Runner-Init>
+L<http://annocpan.org/dist/HPC-Runner>
 
 =item * CPAN Ratings
 
-L<http://cpanratings.perl.org/d/Runner-Init>
+L<http://cpanratings.perl.org/d/HPC-Runner>
 
 =item * Search CPAN
 
-L<http://search.cpan.org/dist/Runner-Init/>
+L<http://search.cpan.org/dist/HPC-Runner/>
 
 =back
 
